"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const Decorators_1 = require("gulpclass/Decorators");
const gulp = require("gulp");
const ts = require("gulp-typescript");
const gulp_tslint_1 = require("gulp-tslint");
const sass = require("gulp-sass");
const nodemon = require("gulp-nodemon");
const del = require("del");
const fs = require("fs");
let YggdrasilGulpfile = class YggdrasilGulpfile {
    constructor() {
        this.tsProject = ts.createProject('tsconfig.json');
        this.tsTestProject = ts.createProject('tsconfig.spec.json');
    }
    start() {
        return ['build', ['watch', 'nodemon']];
    }
    defaultTask() {
        // Default task. Not code is needed.
    }
    build() {
        return ['clean', 'copyAssets', 'tslint', 'compile'];
    }
    buildTest() {
        return ['clean', 'copyAssets', 'tslint:test', 'compile:test'];
    }
    copyAssets() {
        return ['copySass', 'copyJs', 'copyViews', 'copyStatics', 'copyFonts'];
    }
    watch() {
        gulp.watch(['src/**/*.ts'], ['tslint', 'compile', 'nodemon']);
        gulp.watch(['src/public/js/**/*.js'], ['copyJs']);
        gulp.watch(['src/public/scss/**/*.scss'], ['copySass']);
        gulp.watch(['src/views/**/*.pug', 'src/views/**/*.hbs'], ['copyViews']);
    }
    nodemon() {
        return nodemon({
            script: 'dist/ignition.js'
        });
    }
    cleanAll() {
        return del([
            './node_modules',
            './.nyc_output',
            './coverage',
            './package-lock.json',
            './yarn.lock'
        ]);
    }
    clean() {
        return del([
            './dist',
            './logs'
        ]);
    }
    copyStatics() {
        return gulp.src([
            'src/public/images/**/*'
        ]).pipe(gulp.dest('dist/public/images'));
    }
    copyFonts() {
        return gulp.src([
            'src/public/fonts/**/*'
        ]).pipe(gulp.dest('dist/public/fonts'));
    }
    copyViews() {
        return gulp.src([
            'src/views/**/*'
        ]).pipe(gulp.dest('dist/views'));
    }
    copyJs() {
        return gulp.src([
            // Third-party js
            'node_modules/bootstrap/dist/js/bootstrap.min.js',
            'node_modules/jquery/dist/jquery.min.js',
            'node_modules/tether/dist/js/tether.min.js',
            // App js
            'src/public/js/main.js'
        ]).pipe(gulp.dest('dist/public/js'));
    }
    copySass() {
        return gulp.src([
            // Third-party styles
            'node_modules/bootstrap/scss/bootstrap.scss',
            // App styles
            'src/public/scss/*.scss'
        ]).pipe(sass())
            .pipe(gulp.dest('dist/public/css'));
    }
    // TODO: Test this method
    addYggdrasilScriptsToPkg() {
        const parentPkg = require('./node_modules/@yggdrasilts/devs/parent-pkg/parent-pkg.json');
        const projectPkg = require('./package.json');
        fs.writeFileSync('./package.json.bkp', JSON.stringify(projectPkg, null, 2));
        Object.assign(projectPkg.scripts, parentPkg.scripts);
        Object.assign(projectPkg.nyc, parentPkg.nyc);
        fs.writeFileSync('./package.json', JSON.stringify(projectPkg, null, 2));
    }
    /** Non Testing Zone */
    tsLint() {
        return gulp.src(['src/**/*.ts', '!src/**/*.spec.ts'])
            .pipe(gulp_tslint_1.default({ configuration: 'tslint.json' }))
            .pipe(gulp_tslint_1.default.report());
    }
    typescript() {
        return this.tsProject.src()
            .pipe(this.tsProject())
            .js.pipe(gulp.dest('dist'));
    }
    /** Testing Zone */
    tsLintTest() {
        return gulp.src(['src/**/*.ts'])
            .pipe(gulp_tslint_1.default({ configuration: 'tslint.json' }))
            .pipe(gulp_tslint_1.default.report());
    }
    typescriptTest() {
        return this.tsTestProject.src()
            .pipe(this.tsTestProject())
            .js.pipe(gulp.dest('dist'));
    }
};
__decorate([
    Decorators_1.SequenceTask(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "start", null);
__decorate([
    Decorators_1.Task('default', ['build']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "defaultTask", null);
__decorate([
    Decorators_1.SequenceTask(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "build", null);
__decorate([
    Decorators_1.SequenceTask('build:test'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "buildTest", null);
__decorate([
    Decorators_1.SequenceTask(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "copyAssets", null);
__decorate([
    Decorators_1.Task('watch'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "watch", null);
__decorate([
    Decorators_1.Task(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "nodemon", null);
__decorate([
    Decorators_1.Task('clean:all', ['clean']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "cleanAll", null);
__decorate([
    Decorators_1.Task(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "clean", null);
__decorate([
    Decorators_1.Task(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "copyStatics", null);
__decorate([
    Decorators_1.Task(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "copyFonts", null);
__decorate([
    Decorators_1.Task(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "copyViews", null);
__decorate([
    Decorators_1.Task(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "copyJs", null);
__decorate([
    Decorators_1.Task(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "copySass", null);
__decorate([
    Decorators_1.Task('addYggdrasilScriptsToPkg'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "addYggdrasilScriptsToPkg", null);
__decorate([
    Decorators_1.Task('tslint'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "tsLint", null);
__decorate([
    Decorators_1.Task('compile'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "typescript", null);
__decorate([
    Decorators_1.Task('tslint:test'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "tsLintTest", null);
__decorate([
    Decorators_1.Task('compile:test'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], YggdrasilGulpfile.prototype, "typescriptTest", null);
YggdrasilGulpfile = __decorate([
    Decorators_1.Gulpclass()
], YggdrasilGulpfile);
exports.YggdrasilGulpfile = YggdrasilGulpfile;
